package com.sf.vertx.handle;

import org.apache.commons.lang3.StringUtils;

import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.constans.SacErrorCode;

import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class ParameterCheckHandlerImpl implements ParameterCheckHandler {

	@Override
	public void handle(RoutingContext rc) {
		try {
			log.info("rc.request().host():{}", rc.request().host());
			String appCode = rc.request().headers().get(AppConfigHandler.getAppCodeHeaderKey());
			String apiCode = rc.request().headers().get(AppConfigHandler.getApiCodeHeaderKey());
			String key = appCode + ":" + apiCode;
			if (StringUtils.isBlank(appCode) || StringUtils.isBlank(apiCode)
					|| AppConfigHandler.getAppConfig(appCode) == null
					|| AppConfigHandler.getApicodeConfig(key) == null) {
				// 判断OPEN模式, header不传递参数, 走域名映射
				AppConfig appConfig = AppConfigHandler.getAppConfigByDomain(rc.request().host());
				if(appConfig != null) {
					appCode = appConfig.getAppCode();
					apiCode = rc.request().host();
					rc.request().headers().add(AppConfigHandler.getAppCodeHeaderKey(), appCode);
					rc.request().headers().add(AppConfigHandler.getApiCodeHeaderKey(), apiCode);
					key = appCode + ":" + apiCode;
				} else {
					rc.fail(new HttpException(10012));
					return;
				}
			}
			
			if(AppConfigHandler.isDisabledAppcode(apiCode)) {
				rc.fail(new HttpException(10001));
				return;
			}
			
			// 通过请求模式来区分
			String uri = rc.request().uri();
			String httpMethod = rc.request().method().name();
			if(AppConfigHandler.isServiceTypeOpen(key) && AppConfigHandler.isApicodeUri(key, uri, httpMethod) == false) {
				rc.fail(new HttpException(10018));
				return;
			}
			
			// mock
			String mockResponse = AppConfigHandler.mock(key);
			if(StringUtils.isNotBlank(mockResponse)) {
				rc.fail(new MockException(10020, mockResponse));
				return;
			}
		} catch (Exception e) {
			e.printStackTrace();
			rc.fail(new HttpException(SacErrorCode.DEFAULT_ERROR_CODE));
			return;
		}
		
		
		rc.next();
		return;
	}
}
